#!/bin/bash

set -xuv

EXPID=$1
yyyymm=$2
NMDAYS=$3

file=fcdata

rm -f ${EXPID}_???_${yyyymm}01-${yyyymm}${NMDAYS}.nc

echo write \"${file}_[param].grb\"\; > rules_file
grib_filter rules_file ${file} #${file}_${param}.grb, 6h data

#monthly 00GMT: extract good timesteps
for param in 141 142 143 146 147 168 169 175 176 177 179 201 202 243 #212
do
 cdo seltimestep,$(seq -s ',' 1 4 $((NMDAYS*4))) ${file}_${param}.128.grb  ${file}_${param}_mon0.grb 
done

#split daily
for param in 142 143 169 177 179 201 202
do
 i=1
 while [[ $i -le $NMDAYS ]]
 do 
  cdo seltimestep,$(seq -s ',' $(( $i * 4 - 3)) $(( $i * 4 ))) ${file}_${param}.128.grb  ${file}_${param}_d_$i.grb
  i=`expr $i + 1 ` 
 done  
done

#mon0: conv2nc + mean
for infile in `ls *mon0.grb`
do
  cdo -R -r -t ecmwf -f nc splitvar $infile rg
  cdo -f nc -r -timmean rg* ${infile%???}nc 
  rm -f rg*
done
#  cdo -f nc -r -timmean $infile ${infile%???}nc 

#monthly means with 6h
for param in 151 164 165 166 167 168
do
 cdo -R -r -t ecmwf -f nc splitvar ${file}_${param}.128.grb rg
 cdo -f nc -r -timmean rg* ${file}_${param}_mon.nc
 rm -f rg*
done
# cdo -f nc -r -timmean ${file}_${param}.128.grb ${file%????}_${param}_mon.nc

#daily: mean+conv2nc+concat
for param in 142 143 169 177 179 201 202 
do
 for infile in `ls *${param}_d_*.grb`
 do
  cdo -R -r -t ecmwf -f nc splitvar $infile rg
  cdo -f nc -r -timmean rg* ${infile%???}nc 
  rm -f rg*
 done
  i=1
  rm -f l
 while [[ $i -le $NMDAYS ]]
 do
  ls *${param}_d_${i}.nc >> l
  i=`expr $i + 1 `
 done
 ncrcat `cat l` ${file}_${param}_d.nc
#before ncrcat *${param}_d_*.nc  ${file}_${param}_d.nc
# ncrcat ${infile%???}nc  ${file}_${param}_d.nc
done
rm -f *_d_*
#  cdo -f nc -r -timmean $infile ${infile%???}nc 

#6hour
for param in 151 164 165 166 167 168
do
 cdo -R -r -t ecmwf -f nc splitvar ${file}_${param}.128.grb rg
 cdo -f nc copy rg* ${file}_${param}_6h.nc
 rm -f rg*
done
# cdo -f nc copy ${file}_${param}.128.grb ${file}_${param}_6h.nc

#concat all vars in one file for mon,day,6h
for infile in `ls *mon*.nc`
do
 ncks -A $infile ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc
done
 ncap2 -s "TP=(LSP+CP)*1000." ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc2
 mv ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc2 ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc 
 ncatted -O -a units,TP,m,c,"kg m-2 s-1" ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc
 for var in SSHF SLHF SSRD STR STRD TTR SSR TP
 do
  pst=`ncdump -h ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc | grep $var | head -n1 | cut -c 2`
  if [ ! -z $pst ]; then
   ncap2 -s "$var=$var/(3600*6)" ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc2
   mv ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc2 ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc
   ncatted -O -a units,$var,m,c,"W m-2" ${EXPID}_mon_${yyyymm}01-${yyyymm}${NMDAYS}.nc
  fi
  pst=''
 done

for infile in `ls *d.nc`
do
 ncks -A $infile ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc 
done
 ncap2 -s "TP=(LSP+CP)*1000." ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc2
 mv ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc2 ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc
 ncatted -O -a units,TP,m,c,"kg m-2 s-1" ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc
 for var in SSRD STR STRD TTR TP
 do
  pst=`ncdump -h ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc | grep $var | head -n1 | cut -c 2`
  if [ ! -z $pst ]; then
   ncap2 -s "$var=$var/(3600*6)" ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc2
   mv ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc2 ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc
   ncatted -O -a units,$var,m,c,"W m-2" ${EXPID}_day_${yyyymm}01-${yyyymm}${NMDAYS}.nc
  fi
  pst=''
 done

for infile in `ls *6h*.nc`
do
 ncks -A $infile ${EXPID}_6hr_${yyyymm}01-${yyyymm}${NMDAYS}.nc 
done
 for var in SSHF SLHF SSRD STR STRD TTR 
 do
  pst=`ncdump -h ${EXPID}_6hr_${yyyymm}01-${yyyymm}${NMDAYS}.nc | grep $var | head -n1 | cut -c 2`
  if [ ! -z $pst ];then
   ncap2 -s "$var=$var/(3600*6)" ${EXPID}_6hr_${yyyymm}01-${yyyymm}${NMDAYS}.nc ${EXPID}_6hr_${yyyymm}01-${yyyymm}${NMDAYS}.nc2
   mv ${EXPID}_6hr_${yyyymm}01-${yyyymm}${NMDAYS}.nc2 ${EXPID}_6hr_${yyyymm}01-${yyyymm}${NMDAYS}.nc
   ncatted -O -a units,$var,m,c,"W m-2" ${EXPID}_6hr_${yyyymm}01-${yyyymm}${NMDAYS}.nc
  fi
  pst=''
 done

rm -f *grb *mon0* ${file}*nc

