#!/bin/sh
###############################################################################
#                   POSTPROCESSING ECEARTH3 EXPERIMENT
###############################################################################
#
#BSUB -J test
#BSUB -oo test_%J.out
#BSUB -eo test_%J.err
#BSUB -W 04:00
#BSUB -n 32
#BSUB -R "span[ptile=16]"
#
###############################################################################

# Copy the model output in the same directory where test.cmd is located (for nemo rebuilt output is needed)
# remove *.cmor if exist
# nohup ./test.cmd >& test.log &

module load intel/13.0.1
module load hdf5/1.8.10
module load szip/2.1
module load netcdf/4.1.3
module load NCO/4.2.3
PATH=/apps/CDO/1.5.9/bin:${PATH}
PATH=/gpfs/projects/ecm86/opt/grib_api-1.9.9/intel-13/bin:${PATH}

set -xuv
date

#
# General Stuff
#
EXPID=h008
START_date=19930501
MEMBER=fc0
Chunk_start_date=19930501
Chunk_end_date=19930831
RUN_months=4
output_dir=$(pwd)

#
# Cmorization of ocean and atmos.'s output
#
cd $output_dir
CMORIZATION=TRUE
CMORFAMILY=seaIceClimInit
CMORFAMILYSERIES=series22
SPECS2CMOR=/gpfs/projects/ecm86/models/specs2cmor-1.0
MODEL=ecearth_nc
if [[ $CMORIZATION == 'TRUE' && ! -a $output_dir/cmor ]]; then
   experiment_id="${CMORFAMILY}$(echo $START_date | cut -c1-4)"
   real=$(($(echo $MEMBER | cut -d 'c' -f 2)+1))
   init=1
   phys=1
   reftime=$(echo `echo $START_date | cut -c1-4 ``echo "-"``echo $START_date | cut -c5-6``echo "-"``echo $START_date | cut -c7-8``echo "(T00:00:00Z)"`) # yyyy-mm-dd(Thh:mm:ssZ)
#   reftime=$(echo $START_date | cut -c1-6) # yyyymm
   PATH=$SPECS2CMOR/models/$MODEL/bin:${PATH}
   # ocean
   date
   if [[ ! -a $output_dir/cmoro ]]; then
      #available: rebuilt ocean output in *.nc (depends on iodef.xml defined)
      cmoroi=$output_dir/cmoro_input
      cmoroo=$output_dir/cmoro_output
      mkdir -p $cmoroi $cmoroo
      cd $cmoroi
      ln -sf $output_dir/${EXPID}_??_*.nc .
      ln -sf $SPECS2CMOR/common/Tables .
      startyear=$(echo $Chunk_start_date | cut -c1-4)
      endyear=$(echo $Chunk_end_date | cut -c1-4)
      startmonth=$(echo $Chunk_start_date | cut -c5-6)
      endmonth=$(echo $Chunk_end_date | cut -c5-6)
      i=1
      while [ $i -le $RUN_months ]; do
       ncra -d time_counter,$((($i-1)*$RUN_months+1)),$(($i*$RUN_months)) ${EXPID}_1d_${Chunk_start_date}_${Chunk_end_date}_diaptr.nc tmp${i}.nc
       i=$(($i+1))
      done
      rm -f ${EXPID}_1m_${Chunk_start_date}_${Chunk_end_date}_diaptr.nc 
      ncrcat tmp* ${EXPID}_1m_${Chunk_start_date}_${Chunk_end_date}_diaptr.nc
      rm tmp*
cat << EOF > namelist_cmoro
&NAMEXP
 inpath='$cmoroi/',
 outpath='$cmoroo/',
 expname='$EXPID',
 experiment_id=$experiment_id,
 source='EC-EARTH 3.0.1 official release from SMHI',
 model_id='EC-EARTH',
 ecearth=3,
 forcing='N/A',
 parent_experiment_id='piControl',
 parent_experiment_rip='r$((real+1))i${init}p${phys}',
 institution='IC3',
 realization=$real,
 history='',
 comment='',
 references='',
 contact='PA Bretonnière <pierre-antoine.bretonniere@ic3.cat>',
 startyear=$startyear,
 endyear=$endyear,
 startmonth=$startmonth,
 endmonth=$endmonth,
 initialization_method=$init,
 initialization_description='N/A',
 physics_version=$phys,
 physics_description='N/A',
 associated_model='N/A',
 forecast_reference_time='$reftime',
 prefix='$EXPID',
 series=$CMORFAMILYSERIES
/
EOF
      # actual run
      cmor-oce.exe namelist_cmoro
      # post cmorization steps
      cd $cmoroo
      tarfile="$output_dir/CMORO_${EXPID}_${START_date}_${MEMBER}_${Chunk_start_date}-${Chunk_end_date}.tar"
      tar -cvf $tarfile *
      gzip -9 -f $tarfile
      rm -rf $cmoroi $cmoroo
      touch $output_dir/cmoro
   fi
   # atmosphere
   date
   if [[ ! -a $output_dir/cmora ]]; then
      # available: daily grib (3h or 6h) on monthly basis
      NFRP=6
      cmorai=$output_dir/cmora_input
      cmorao=$output_dir/cmora_output
      mkdir -p $cmorai $cmorao
      cd $cmorai
      ln -sf $SPECS2CMOR/common/Tables .
      NYEAR0=$(echo $START_date | cut -c1-4)
      NMONTH0=$(echo $START_date | cut -c5-6)
      csy=$(echo $Chunk_start_date | cut -c1-4) # chunk start year
      csm=$(echo $Chunk_start_date | cut -c5-6) # chunk start month
      cs=$RUN_months # chunk size (in months)
      for c in $(seq 1 $cs); do
       csm=${csm#0}
       list[$c]="${csy}$(printf "%02d" $csm)"
       div=$((((csm+1))/13))
       rem=$((((csm+1))%13))
       csy=$((csy+div))
       csm=$rem
       if [[ $rem -eq 0 ]]; then
        csm=01
       fi
      done
      echo ${list[@]}
      for yyyymm in ${list[@]}; do
       file=fcdata
       rm -f fcdata fcdata0
       rm -rf tmp_dir_means
       mkdir tmp_dir_means
       cd tmp_dir_means
       cdo sp2gpl $output_dir/ICMSH${EXPID}+${yyyymm} fcdata0
       cat fcdata0 $output_dir/ICMGG${EXPID}+${yyyymm}  > $file
       cp fcdata /gpfs/projects/ecm86/models/specs2cmor-1.0/models/ecearth_nc/test/input_atm/fcdata_${yyyymm}
       rm fcdata0
       NYEAR=$(echo $yyyymm | cut -c1-4)
       NMONTH=$(echo $yyyymm | cut -c5-6)
       NMDAYS=$(cal $NMONTH $NYEAR | grep -v "[a-z]" | grep -v "^$" | wc -w)
       date
#debug       /gpfs/projects/ecm86/models/specs2cmor-1.0/models/ecearth_nc/test/old_mn_means.sh $EXPID $yyyymm $NMDAYS
#debug       /gpfs/scratch/ecm86/ecm86010/test/cmor_nc/means.sh $EXPID $yyyymm $NMDAYS
        /gpfs/projects/ecm86/models/specs2cmor-1.0/models/ecearth_nc/test/new_means.sh $EXPID $yyyymm $NMDAYS
#        cp ${EXPID}_*_${yyyymm}01-${yyyymm}${NMDAYS}.nc /gpfs/projects/ecm86/models/specs2cmor-1.0/models/ecearth_nc/test/input_atm/.
        mv ${EXPID}_*_${yyyymm}01-${yyyymm}${NMDAYS}.nc ../.
       cd ..
       date
       if [[ $NYEAR == 1850 && $NMONTH == 01 ]]; then
        suff=1
       else
        suff=""
       fi
cat << EOF > namelist_cmora
&NAMEXP
 inpath='$cmorai/',
 outpath='$cmorao/',
 expname='$EXPID',
 experiment_id=$experiment_id,
 source='EC-EARTH 3.0.1 official release from SMHI',
 model_id='EC-EARTH',
 forcing='N/A',
 parent_experiment_id='piControl',
 parent_experiment_rip='r$((real+1))i${init}p${phys}',
 institution='IC3',
 realization=$real,
 history='',
 comment='',
 references='',
 contact='PA Bretonnière <pierre-antoine.bretonniere@ic3.cat>',
 startyear=$NYEAR,
 endyear=$NYEAR,
 startmonth=$NMONTH,
 endmonth=$NMONTH,
 initialization_method=$init,
 initialization_description='N/A',
 physics_version=$phys,
 physics_description='N/A',
 associated_model='N/A',
 forecast_reference_time='$reftime',
 prefix='$EXPID',
 series=$CMORFAMILYSERIES
/
EOF
       # actual run
       cmor-atm.exe namelist_cmora
      done
      unset list
      # post cmorization steps
      cd $cmorao
      tarfile="$output_dir/CMORA_${EXPID}_${START_date}_${MEMBER}_${Chunk_start_date}-${Chunk_end_date}.tar"
      tar -cvf $tarfile *
      gzip -9 -f $tarfile
      rm -rf $cmorai $cmorao
      touch $output_dir/cmora
   fi
   if [[ -a $output_dir/cmoro && -a $output_dir/cmora ]]; then
      rm $output_dir/cmoro $output_dir/cmora
      touch $output_dir/cmor
      date
   fi
fi

date

